import { CrysteamPortal, Methods, RoutePermission } from "@crysteam/portal";
import { AxiosInstance } from "axios";

export type NotifyOptions = {
    portal: CrysteamPortal;
    scopes?: RoutePermission[];
    host?: string;
}

export type NotificationPayload = {
    user_id: string;
    title?: string;
    message: string;
}

export class Notify {
    private _http?: AxiosInstance;
    private readonly _host: string;
    private readonly _portal: CrysteamPortal;

    private _availableScopes?: RoutePermission[];
    private readonly _customScopes?: RoutePermission[];

    constructor(options: NotifyOptions) {
        this._portal = options.portal;
        this._host = options.host || "https://notify.crysteam.ru";
        this._customScopes = options.scopes;
        this._initialize();
    }

    private async _initialize(): Promise<void> {
        const domain: string = new URL(this._host).hostname;
        this._availableScopes = this._customScopes || await this._portal.getServiceScopesFor(domain);
        this._http = this._portal.createHttpClientFor(this._host, this._availableScopes as any);
    }

    private _hasScope(scope: RoutePermission): boolean {
        return !!(this._availableScopes && this._portal.isScopeAllowed(this._availableScopes, scope));
    }

    async send(payload: NotificationPayload): Promise<void> {
        if (!this._http || !this._hasScope({ method: Methods.POST, path: "/notify" })) {
            throw new Error("Insufficient permission to notify user.");
        }

        await this._http.post("/notify", payload);
    }

}
